from dataclasses import dataclass

from .property import Property


@dataclass
class Rollup(Property):
    """Rollup class

    ex.
    {'id': '%3BXyX', 'type': 'rollup', 'rollup': {'type': 'number', 'number': 0, 'function': 'count_values'}}
    """

    _rollup: dict

    def __init__(
        self,
        name: str,
        rollup: dict = {},
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self._rollup = rollup
        self.id = id

    @staticmethod
    def of(key: str, param: dict) -> "Rollup":
        return Rollup(
            id=param["id"],
            name=key,
            rollup=param["rollup"],
        )

    @property
    def value(self) -> dict:
        rollup_type = self._rollup["type"]
        return self._rollup[rollup_type]

    @property
    def type(self) -> str:
        return "rollup"

    def __dict__(self) -> dict:
        raise NotImplementedError("this dict method must not be called")

    def value_for_filter(self) -> str:
        raise NotImplementedError




from typing import Any

from .properties.button import Button
from .properties.checkbox import Checkbox
from .properties.created_by import CreatedBy
from .properties.created_time import CreatedTime
from .properties.date import Date
from .properties.email import Email
from .properties.formula import Formula
from .properties.last_edited_by import LastEditedBy
from .properties.last_edited_time import LastEditedTime
from .properties.multi_select import MultiSelect
from .properties.number import Number
from .properties.person import People
from .properties.phone_number import PhoneNumber
from .properties.properties import Properties
from .properties.property import Property
from .properties.relation import Relation
from .properties.rollup import Rollup
from .properties.select import Select
from .properties.status import Status
from .properties.text import Text
from .properties.title import Title
from .properties.unique_id import UniqueId
from .properties.url import Url


class PropertyTranslator:
    @staticmethod
    def from_dict(properties: dict[str, dict]) -> Properties:
        values = []
        for key, value in properties.items():
            values.append(PropertyTranslator.from_property_dict(key, value))
        return Properties(values=[value for value in values if value is not None])

    @staticmethod
    def from_property_dict(key: str, property_: dict[str, Any]) -> "Property":  # noqa: PLR0911
        type_ = property_["type"]
        match type_:
            case "title":
                return Title.from_property(key, property_)
            case "rich_text":
                return Text.from_dict(key, property_)
            case "multi_select":
                return MultiSelect.of(key, property_)
            case "select":
                return Select.of(key, property_)
            case "number":
                return Number.of(key, property_)
            case "checkbox":
                return Checkbox.of(key, property_)
            case "date":
                return Date.of(key, property_)
            case "status":
                return Status.of(key, property_)
            case "url":
                return Url.of(key, property_)
            case "relation":
                return Relation.of(key, property_)
            case "last_edited_time":
                return LastEditedTime.create(key, property_["last_edited_time"])
            case "created_time":
                return CreatedTime.create(key, property_["created_time"])
            case "rollup":
                return Rollup.of(key, property_)
            case "button":
                return Button.of(key, property_)
            case "people":
                return People.of(key, property_)
            case "email":
                return Email.of(key, property_)
            case "phone_number":
                return PhoneNumber.of(key, property_)
            case "created_by":
                return CreatedBy.of(key, property_)
            case "last_edited_by":
                return LastEditedBy.of(key, property_)
            case "formula":
                return Formula.of(key, property_)
            case "unique_id":
                return UniqueId.of(key, property_)
            case _:
                msg = f"Unsupported property type: {type_} {property_}"
                raise Exception(msg)



import re
from dataclasses import dataclass


@dataclass(frozen=True)
class PageId:
    _value: str

    def __post_init__(self) -> None:
        # 型チェック
        if not isinstance(self._value, str):
            msg = f"page_idは文字列である必要があります: {self._value}"
            raise TypeError(msg)

        # UUID4の形式であることを確認する
        if not re.match(
            r"[0-9a-f]{8}-?[0-9a-f]{4}-?[0-9a-f]{4}-?[0-9a-f]{4}-?[0-9a-f]{12}",
            self._value,
        ):
            msg = f"page_idの形式が不正です: {self._value}"
            raise ValueError(msg)

    @staticmethod
    def dummy() -> "PageId":
        return PageId(_value="5c38fd30-714b-4ce2-bf2d-25407f3cfc16")

    @property
    def value(self) -> str:
        """UUID4の形式の文字列を返す"""
        # まずハイフンを削除してから、ハイフンをつけなおす
        _value = self._value.replace("-", "")
        return "-".join([_value[:8], _value[8:12], _value[12:16], _value[16:20], _value[20:]])



from enum import Enum


class Cond(Enum):
    EQUALS = "equals"
    DOES_NOT_EQUAL = "does_not_equal"
    IS_EMPTY = "is_empty"
    IS_NOT_EMPTY = "is_not_empty"
    CONTAINS = "contains"
    DOES_NOT_CONTAIN = "does_not_contain"
    GREATER_THAN = "greater_than"
    GREATER_THAN_OR_EQUAL_TO = "greater_than_or_equal_to"
    LESS_THAN = "less_than"
    LESS_THAN_OR_EQUAL_TO = "less_than_or_equal_to"
    STARTS_WITH = "starts_with"
    ENDS_WITH = "ends_with"
    AFTER = "after"
    ON_OR_AFTER = "on_or_after"
    BEFORE = "before"
    ON_OR_BEFORE = "on_or_before"
    NEXT_MONTH = "next_month"
    NEXT_WEEK = "next_week"
    NEXT_YEAR = "next_year"
    PAST_MONTH = "past_month"
    PAST_WEEK = "past_week"
    PAST_YEAR = "past_year"
    THIS_WEEK = "this_week"
    CHECKBOX = "checkbox"
    DATE = "date"
    NUMBER = "number"
    STRING = "string"
    ANY = "any"
    EVERY = "every"
    NONE = "none"



import json
from unittest import TestCase

import pytest
from lotion.base_page import BasePage
from lotion.properties.title import Title


class TestBasePage(TestCase):
    def test_ページを作成する(self):
        # When
        actual = BasePage.create(properties=[], blocks=[])

        # Then
        self.assertEqual([], actual.properties.values)

    def test_タイトルとリンクをSlack形式で出力する(self):
        # isinstanceのためにパスを揃える
        import sys

        sys.path.append("notion_api")

        # Given
        base_page = BasePage.create(
            properties=[Title.from_plain_text(name="名前", text="タイトル")],
        )
        base_page.update_id_and_url(page_id="dummy-id", url="http://example.com")

        # When
        actual = base_page.title_for_slack()

        # Then
        self.assertEqual("<http://example.com|タイトル>", actual)

    @pytest.mark.current()
    def test_webhookからのリクエストボディを処理できる(self):
        given = json.load(open("test/base_page_test/pattern1.json"))
        print(given)

        actual = BasePage.from_data(given)
        print(actual)
        self.assertEqual(given["id"], actual.id)



from typing import Optional
from dataclasses import dataclass
from lotion.block.rich_text.rich_text_element import RichTextElement, RichTextTextElement, RichTextMentionElement
from lotion.block.rich_text.rich_text import RichText


@dataclass(frozen=True)
class RichTextBuilder:
    rich_text: list[RichTextElement]

    @staticmethod
    def get_instance() -> "RichTextBuilder":
        return RichTextBuilder(rich_text=[])

    def add_text(self, content: str, link_url: Optional[str] = None) -> "RichTextBuilder":
        self.rich_text.append(RichTextTextElement.of(content, link_url))
        return self

    def add_page_mention(self, page_id: str) -> "RichTextBuilder":
        self.rich_text.append(
            RichTextMentionElement.of_page(page_id=page_id))
        return self

    def build(self) -> RichText:
        return RichText(elements=self.rich_text)



from .block import Block
from .rich_text.rich_text import RichText


class ToDo(Block):
    rich_text: RichText
    color: str
    checked: bool

    def __init__(
        self,
        rich_text: RichText,
        color: str | None = None,
        checked: bool | None = None,
        id: str | None = None,
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ):
        super().__init__(
            id, archived, created_time, last_edited_time, has_children, parent
        )
        self.rich_text = rich_text
        self.color = color
        self.checked = checked or False

    @staticmethod
    def of(block: dict) -> "ToDo":
        to_do = block["to_do"]
        rich_text = RichText.from_entity(to_do["rich_text"])
        return ToDo(
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
            rich_text=rich_text,
            color=to_do["color"],
            checked=to_do["checked"],
        )

    @staticmethod
    def from_plain_text(text: str, checked: bool | None = None) -> "ToDo":
        checked = checked or False
        return ToDo(rich_text=RichText.from_plain_text(text), checked=checked)

    @property
    def type(self) -> str:
        return "to_do"

    def to_dict_sub(self) -> dict:
        result = {
            "rich_text": self.rich_text.to_dict(),
            "checked": self.checked,
        }
        if self.color is not None:
            result["color"] = self.color
        return result

    def to_slack_text(self) -> str:
        return (
            "[ ] " + self.rich_text.to_slack_text()
            if not self.checked
            else "[x] " + self.rich_text.to_slack_text()
        )



from dataclasses import dataclass

from ..block.rich_text.rich_text_element import (
    RichTextMentionElement,
    RichTextTextElement,
)
from ..page.page_id import PageId
from .property import Property


@dataclass
class Title(Property):
    text: str
    value: list[dict]
    type: str = "title"
    mentioned_page_id: str | None = None

    def __init__(
        self,
        name: str,
        id: str | None = None,
        value: list[dict] = [],
        text: str | None = None,
        mentioned_page_id: str | None = None,
    ):
        self.name = name
        self.id = id
        self.value = value
        self.text = text
        self.mentioned_page_id = mentioned_page_id

    @classmethod
    def from_properties(cls, properties: dict) -> "Title":
        if "Name" in properties:
            return cls.__of("Name", properties["Name"])
        if "Title" in properties:
            return cls.__of("Title", properties["Title"])
        if "名前" in properties:
            return cls.__of("名前", properties["名前"])
        msg = f"Title property not found. properties: {properties}"
        raise Exception(msg)

    @classmethod
    def from_property(cls, key: str, property: dict) -> "Title":
        return cls.__of(key, property)

    def __dict__(self) -> dict:
        result = {
            "title": self._get_value(),
        }
        if self.id is not None:
            result["id"] = self.id
        return {
            self.name: result,
        }

    def _get_value(self) -> list[dict]:
        if self.value is not None and self.value != []:
            return self.value
        values = []
        values.append(
            {
                "type": "text",
                "text": {
                    "content": self.text,
                },
            },
        )
        if self.mentioned_page_id is not None:
            values.append(
                {
                    "type": "mention",
                    "mention": {
                        "type": "page",
                        "page": {
                            "id": self.mentioned_page_id,
                        },
                    },
                    # "plain_text": self.text,
                    # "href": f"https://www.notion.so/{self.mentioned_page_id}"
                },
            )
        return values

    @staticmethod
    def __of(name: str, param: dict) -> "Title":
        return Title(
            name=name,
            id=param["id"],
            value=param["title"],
            text="".join([item["plain_text"] for item in param["title"]]),
        )

    @staticmethod
    def from_plain_text(name: str = "名前", text: str = "") -> "Title":
        return Title(
            name=name,
            text=text,
        )

    @staticmethod
    def from_mentioned_page(
        mentioned_page_id: str,
        mentioned_page_title: str,
        name: str = "名前",
        prefix: str = "",
        suffix: str = "",
    ) -> "Title":
        title = prefix + mentioned_page_title + suffix
        values = []
        if prefix != "":
            rich_text_element = RichTextTextElement.of(content=prefix)
            values.append(rich_text_element.to_dict())
        page_id_object = PageId(mentioned_page_id)
        rich_text_element = RichTextMentionElement.from_page_type(page_id_object.value)
        values.append(rich_text_element.to_dict())
        if suffix != "":
            rich_text_element = RichTextTextElement.of(content=suffix)
            values.append(rich_text_element.to_dict())
        return Title(
            name=name,
            value=values,
            text=title,
        )

    @staticmethod
    def from_mentioned_page_id(
        page_id: str,
        name: str = "名前",
    ) -> "Title":
        return Title(
            name=name,
            text="",
            mentioned_page_id=page_id,
        )

    def value_for_filter(self) -> str:
        return self.text



from dataclasses import dataclass
from typing import Any


@dataclass
class Cover:
    type: str
    external_url: str | None = None

    def __init__(
        self, type: str, external_url: str | None = None
    ) -> None:  # noqa: A002
        self.type = type
        self.external_url = external_url

    @staticmethod
    def of(param: dict) -> "Cover":
        return Cover(
            type=param["type"],
            external_url=param["external"]["url"] if "external" in param else None,
        )

    @staticmethod
    def from_external_url(external_url: str) -> "Cover":
        return Cover(
            type="external",
            external_url=external_url,
        )

    def __dict__(self) -> dict:
        result: dict[str, Any] = {
            "type": self.type,
        }
        if self.external_url is not None:
            result["external"] = {
                "url": self.external_url,
            }
        return result

    def value_for_filter(self) -> str:
        raise NotImplementedError



from .block import Block


class Embed(Block):
    caption: list
    embed_url: str

    def __init__(  # noqa: PLR0913
        self,
        embed_url: str,
        caption: list | None = None,
        id: str | None = None,  # noqa: A002
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ) -> None:
        super().__init__(
            id, archived, created_time, last_edited_time, has_children, parent
        )
        self.embed_url = embed_url
        self.caption = caption or []

    @staticmethod
    def from_url(url: str) -> "Embed":
        return Embed(
            embed_url=url,
            caption=[],
        )

    @staticmethod
    def from_url_and_caption(url: str, caption_str: str | None = None) -> "Embed":
        if caption_str is not None:
            raise NotImplementedError
        return Embed(
            embed_url=url,
            caption=[],
        )

    @staticmethod
    def of(block: dict) -> "Embed":
        embed = block["embed"]
        return Embed(
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
            caption=embed["caption"],
            embed_url=embed.get("url", ""),
        )

    @property
    def type(self) -> str:
        return "embed"

    def to_dict_sub(self) -> dict:
        result = {
            "url": self.embed_url,
        }

        if self.caption:
            result["caption"] = self.caption
        return result

    def to_slack_text(self) -> str:
        return self.embed_url
